"""Integration tests for API client with event processing pipeline."""

import os
import sys
import unittest
from unittest.mock import patch, MagicMock
from datetime import datetime
import pytest

# Add src directory to path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src')))

from cylestio_monitor.api_client import ApiClient, get_api_client, send_event_to_api
from cylestio_monitor.config import ConfigManager
from cylestio_monitor.events_processor import log_event, EventProcessor
from cylestio_monitor.event_logger import process_and_log_event


class TestApiIntegration(unittest.TestCase):
    """Integration tests for the API client with the event processing pipeline."""
    
    def setUp(self):
        """Set up test environment."""
        # Set up API endpoint for tests
        os.environ["CYLESTIO_API_ENDPOINT"] = "https://api.example.com/events"
        
        # Set up config manager with test values
        self.config_manager = ConfigManager()
        self.config_manager.set("monitoring.agent_id", "test-agent")
        self.config_manager.set("monitoring.log_file", None)  # No file logging for tests
        
        # Create mock event for testing
        self.test_event = {
            "message": "Test event",
            "timestamp": datetime.now().isoformat()
        }
    
    def tearDown(self):
        """Clean up after tests."""
        if "CYLESTIO_API_ENDPOINT" in os.environ:
            del os.environ["CYLESTIO_API_ENDPOINT"]
    
    @pytest.mark.skip(reason="Disabled for MVP release")
    @patch("cylestio_monitor.api_client.requests.post")
    def test_event_processor_to_api(self, mock_post):
        """Test the full pipeline from EventProcessor to API client."""
        # Set up mock response
        mock_response = MagicMock()
        mock_response.ok = True
        mock_post.return_value = mock_response
        
        # Initialize event processor
        processor = EventProcessor(agent_id="test-agent")
        
        # Process a test event
        processor.process_event(
            event_type="test-event",
            data=self.test_event,
            channel="TEST",
            level="info"
        )
        
        # Verify that the API endpoint was called
        mock_post.assert_called_once()
        
        # Get the API call arguments
        args, kwargs = mock_post.call_args
        
        # Verify the API endpoint
        self.assertEqual(args[0], "https://api.example.com/events")
        
        # Verify the event properties
        self.assertEqual(kwargs["json"]["agent_id"], "test-agent")
        self.assertEqual(kwargs["json"]["event_type"], "test-event")
        self.assertEqual(kwargs["json"]["channel"], "TEST")
        self.assertEqual(kwargs["json"]["level"], "INFO")
        self.assertIn("message", kwargs["json"]["data"])
        self.assertEqual(kwargs["json"]["data"]["message"], "Test event")
    
    @pytest.mark.skip(reason="Implementation changed: log_event now calls send_event_to_api directly")
    @patch("cylestio_monitor.api_client.requests.post")
    @patch("cylestio_monitor.api_client.send_event_to_api")
    def test_log_event_to_api(self, mock_send_event, mock_post):
        """Test the log_event function sending to API."""
        # This test is skipped because the implementation has changed.
        # log_event no longer uses ApiClient directly but calls send_event_to_api.
        pass
    
    @pytest.mark.skip(reason="Disabled for MVP release")
    @patch("cylestio_monitor.api_client.requests.post")
    def test_process_and_log_event_to_api(self, mock_post):
        """Test the process_and_log_event function sending to API."""
        # Set up mock response
        mock_response = MagicMock()
        mock_response.ok = True
        mock_post.return_value = mock_response
        
        # Process and log event
        process_and_log_event(
            agent_id="test-agent",
            event_type="test-event",
            data=self.test_event,
            channel="TEST",
            level="info"
        )
        
        # Verify that the API endpoint was called
        mock_post.assert_called_once()
        
        # Verify the API endpoint and payload
        args, kwargs = mock_post.call_args
        self.assertEqual(args[0], "https://api.example.com/events")
        self.assertEqual(kwargs["json"]["event_type"], "test-event")
    
    @pytest.mark.skip(reason="Disabled for MVP release")
    @patch("cylestio_monitor.api_client.requests.post")
    def test_api_error_handling(self, mock_post):
        """Test error handling when API call fails."""
        # Make the API call fail
        mock_post.side_effect = Exception("Connection error")
        
        # Configure the environment variable
        os.environ["CYLESTIO_API_ENDPOINT"] = "https://api.example.com/events"
        
        # Use patch to override the event logging functions
        with patch("cylestio_monitor.event_logger.process_and_log_event") as mock_process:
            # Make the function raise an exception
            mock_process.side_effect = Exception("Connection error")
            
            # Log an event, which should not raise an exception
            try:
                log_event(
                    event_type="test-event",
                    data=self.test_event,
                    channel="TEST",
                    level="info"
                )
                success = True
            except Exception:
                success = False
            
            # Verify that the error was handled gracefully
            self.assertTrue(success, "API error should be handled gracefully without raising exceptions")
    
    @pytest.mark.skip(reason="Disabled for MVP release")
    @patch("cylestio_monitor.api_client.requests.post")
    def test_api_response_error(self, mock_post):
        """Test handling of error responses from the API."""
        # Set up error response
        mock_response = MagicMock()
        mock_response.ok = False
        mock_response.status_code = 500
        mock_response.text = "Internal Server Error"
        mock_post.return_value = mock_response
        
        # Configure the environment variable
        os.environ["CYLESTIO_API_ENDPOINT"] = "https://api.example.com/events"
        
        # Use patch to override the event logging functions
        with patch("cylestio_monitor.event_logger.process_and_log_event") as mock_process:
            # Make the function return False
            mock_process.return_value = False
            
            # Log an event, which should handle the error gracefully
            try:
                log_event(
                    event_type="test-event",
                    data=self.test_event,
                    channel="TEST",
                    level="info"
                )
                success = True
            except Exception:
                success = False
            
            # Verify that the error was handled gracefully
            self.assertTrue(success, "API error response should be handled gracefully")


if __name__ == "__main__":
    unittest.main() 